@@ -12,7 +12,10 @@ from . import file_utils
 
 EXPECTATIONS = {}
 """
-Global storage for expectations by spec module name and username.
+Global storage for expectations by spec module name, mode, and username.
+Entries are module names with mode dictionaries as values, whose keys are
+modes (i.e., "evaluation" or "validation") and whose values are
+dictionaries that map usernames to lists of expectations.
 """
 
 CURRENT_EXAMPLE = None
@@ -188,6 +191,8 @@ class Expectation:
                None.
         """
         rows_here = report["table"]
+        if rows_here == [] and report["files"] == []:
+            raise ValueError("Report indicates no file to evaluate.")
         found = None
         trail = []
 
@@ -455,25 +460,35 @@ def check_entire_report(
     return (succeeded, explanation)
 
 
-def example(username):
+def example(username, extra_modes=()):
     """
-    Registers a current username such that calls to expect create
-    expectations for that example submission, and creates an entry in the
-    expectations table for it so that even if no expectations are
-    established it will still be tested using default expectations.
+    Registers a current username such that calls to `expect` and/or
+    `expect_validation` create expectations for that example submission,
+    and creates an "evaluation" entry in the expectations table for it so
+    that even if no expectations are established it will still be tested
+    using default expectations.
+
+    If extra_modes is provided, it should be a list of strings naming
+    extra modes to check (e.g., ["validation"]). Note that as soon as an
+    expectation is established for any mode that mode will be checked
+    even if it wasn't specified here.
     """
     global CURRENT_EXAMPLE
     CURRENT_EXAMPLE = username
     mname = file_utils.get_spec_module_name()
-    EXPECTATIONS\
-        .setdefault(mname, {})\
-        .setdefault(username, [])
+    for mode in ["evaluation"] + list(extra_modes):
+        EXPECTATIONS\
+            .setdefault(mname, {})\
+            .setdefault(mode, {})\
+            .setdefault(username, [])
 
 
-def expect(status, *id_or_path):
+def expect(status, *id_or_path, mode="evaluation"):
     """
     Creates an `Expectation` object and registers it under the current
-    example username. Arguments are:
+    example username as an evaluation expectation.
+
+    Arguments are:
 
     - status: The expected status. See `potluck.rubrics.Goal`.
     - id_or_path: One or more additional strings specifying which goal we're
@@ -482,6 +497,9 @@ def expect(status, *id_or_path):
         of a rubric-description-path. If it's a single string, it should
         start with the goal type and then category when using the default
         rubric metric.
+    - mode: Keyword-only; sets which mode of testing the expectation
+        applies to. Valid modes are "evaluation" (the default) and
+        "validation".
     """
     mname = file_utils.get_spec_module_name()
 
@@ -492,6 +510,7 @@ def expect(status, *id_or_path):
 
     EXPECTATIONS\
         .setdefault(mname, {})\
+        .setdefault(mode, {})\
         .setdefault(CURRENT_EXAMPLE, [])\
         .append(
             Expectation(
@@ -501,10 +520,19 @@ def expect(status, *id_or_path):
         )
 
 
-def expect_warnings(fragment=''):
+def expect_validation(*args):
+    """
+    Establishes an expectation for the validation step. This is just a
+    shortcut for calling `expect` with mode set to "validation".
+    """
+    expect(*args, mode="validation")
+
+
+def expect_warnings(fragment='', mode="evaluation"):
     """
     Creates an `ExpectedWarning` object and registers it under the
-    current example username.
+    current example username. Registers for evaluation by default, but
+    you can specify a different mode (e.g., "validation").
 
     The `fragment` argument if omitted will cause all warnings to be
     treated as expected, but if provided, only warnings whose raw HTML
@@ -515,13 +543,16 @@ def expect_warnings(fragment=''):
 
     EXPECTATIONS\
         .setdefault(mname, {})\
+        .setdefault(mode, {})\
         .setdefault(CURRENT_EXAMPLE, [])\
         .append(ExpectedWarning(fragment))
 
 
-def get_expectations(spec_module):
+def get_expectations(spec_module, mode="evaluation"):
     """
-    Returns all expectations for the given specification module, as a
-    dictionary mapping user IDs to expectation lists.
+    Returns all expectations for the given specification module and mode,
+    as a dictionary mapping user IDs to expectation lists. Returns None
+    if there are no expectations for the target mode or for the target
+    module, and an empty expectation set hasn't been set up either.
     """
-    return EXPECTATIONS.get(spec_module.__name__, {})
+    return EXPECTATIONS.get(spec_module.__name__, {}).get(mode, None)
