@@ -632,7 +632,7 @@ def set_extension(
 
     If only_from is provided, the operation will fail when the extension
     value being updated isn't set to that value (may be a number of
-    hours, or True for the standared extension, or False for unset). In
+    hours, or True for the standard extension, or False for unset). In
     that case, this function will return False if it fails. Set
     only_from to None to unconditionally update the extension.
     """
@@ -1433,6 +1433,8 @@ def fetch_best_outcomes(course, semester, username):
     - "submitted_at" - A time string (see
         `potluck.time_utils.timestring`) indicating when the list of
         outcomes was submitted.
+    - "on_time" - A boolean indicating whether the submission came in on
+        time or not.
     - "authors" - A list of usernames for participating authors.
     - "outcomes" - A list of 3-tuple outcomes, which contain a boolean
         for success/failure followed by tag and message strings (see
@@ -1501,7 +1503,9 @@ def save_outcomes(
     outcomes,
     codeBlocks,
     status,
-    credit
+    credit,
+    group_credit,
+    on_time
 ):
     """
     Saves a list of outcomes for a specific exercise submitted by a
@@ -1516,7 +1520,9 @@ def save_outcomes(
     be a string describing the status of the submission, while the credit
     value should be a number where a higher number indicates a better
     submission (to be stored preferentially as the "best" submission so
-    far if it's better than the old one).
+    far if it's better than the old one). The `on_time` value should be a
+    boolean indicating whether the outcomes should count as 'on-time' or
+    not.
 
     The list of outcomes and associated code blocks is added to the
     record of all such lists submitted for that exercise by that user, as
@@ -1524,11 +1530,13 @@ def save_outcomes(
 
     - "submitted_at": the current time, as a string (see
         `potluck.time_utils.timestring`).
+    - "on_time": The `on_time` boolean provided.
     - "authors": The list of authors.
     - "outcomes": The list of outcomes.
     - "code": The list of filename/code-string pairs.
     - "status": The status string.
     - "credit": The credit number.
+    - "group_credit": The credit number for counting group credit.
 
     It will also replace any old outcomes list which is listed in the
     best-outcomes-per-exercise info for the submitting user if it's
@@ -1558,11 +1566,13 @@ def save_outcomes(
     # Here's the info we store
     info = {
         "submitted_at": timestring,
+        "on_time": on_time,
         "authors": authors,
         "outcomes": outcomes,
         "code": codeBlocks,
         "status": status,
-        "credit": credit
+        "credit": credit,
+        "group_credit": group_credit
     }
 
     recorded_outcomes.append(info)
@@ -1596,7 +1606,11 @@ def save_outcomes(
             old_credit = -1
         else:
             old_best = json.loads(old_json)
-            old_credit = old_best.get("credit", -1)
+            old_credit = old_best.get(
+                "group_credit",
+                # for older data, group_credit isn't present
+                old_best.get("credit", -1)
+            )
     except Exception:
         flask.flash("Failed to read best-outcomes info!")
         tb = potluck.html_tools.string_traceback()
@@ -1610,12 +1624,12 @@ def save_outcomes(
         return False
 
     # Missing credit value counts as -1 credit.
-    if credit is None:
-        credit = -1
+    if group_credit is None:
+        group_credit = -1
 
     # Only record a new best-outcome-for-this-exercise if the current
-    # credit value is >= the old one.
-    if credit >= old_credit:
+    # group credit value is >= the old one.
+    if group_credit >= old_credit:
         try:
             _REDIS.hset(best_key, exercise, json.dumps(info))
         except Exception:
